@@ -1,65 +1,25 @@
-#=======================================================================
-#    ____  ____  _____              _    ____ ___   ____
-#   |  _ \|  _ \|  ___|  _   _     / \  |  _ \_ _| |___ \
-#   | |_) | | | | |_    (_) (_)   / _ \ | |_) | |    __) |
-#   |  __/| |_| |  _|    _   _   / ___ \|  __/| |   / __/
-#   |_|   |____/|_|     (_) (_) /_/   \_\_|  |___| |_____|
-#
-#   A Perl Module Chain to faciliate the Creation and Modification
-#   of High-Quality "Portable Document Format (PDF)" Files.
-#
-#   Copyright 1999-2005 Alfred Reibenschuh <areibens@cpan.org>.
-#
-#=======================================================================
-#
-#   This library is free software; you can redistribute it and/or
-#   modify it under the terms of the GNU Lesser General Public
-#   License as published by the Free Software Foundation; either
-#   version 2 of the License, or (at your option) any later version.
-#
-#   This library is distributed in the hope that it will be useful,
-#   but WITHOUT ANY WARRANTY; without even the implied warranty of
-#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-#   Lesser General Public License for more details.
-#
-#   You should have received a copy of the GNU Lesser General Public
-#   License along with this library; if not, write to the
-#   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-#   Boston, MA 02111-1307, USA.
-#
-#   $Id: FontFile.pm,v 1.14 2005/03/20 23:24:23 fredo Exp $
-#
-#=======================================================================
 package PDF::API2::Resource::CIDFont::TrueType::FontFile;
 
-BEGIN {
+our $VERSION = '2.020'; # VERSION
 
-    use utf8;
-    use Encode qw(:all);
-    use PDF::API2::Util;
-    use PDF::API2::IOString;
+use base 'PDF::API2::Basic::PDF::Dict';
 
-    use PDF::API2::Basic::PDF::Utils;
-    use PDF::API2::Basic::PDF::Dict;
-    use PDF::API2::Basic::TTF::Font;
+use Encode qw(:all);
+use Font::TTF::Font;
+use POSIX qw(ceil floor);
 
-    use POSIX;
+use PDF::API2::Util;
+use PDF::API2::Basic::PDF::Utils;
 
-    use vars qw( @ISA $VERSION $cmap );
-
-    @ISA = qw( PDF::API2::Basic::PDF::Dict );
-
-    ( $VERSION ) = '$Revision: 1.14 $' =~ /Revision: (\S+)\s/; # $Date: 2005/03/20 23:24:23 $
-    $cmap={};
-}
 no warnings qw[ recursion uninitialized ];
 
+our $cmap = {};
 
 sub _look_for_cmap ($) {
     my $fname=lc(shift);
     $fname=~s/[^a-z0-9]+//gi;
     return({%{$cmap->{$fname}}}) if(defined $cmap->{$fname});
-    eval "require PDF::API2::Resource::CIDFont::CMap::$fname; ";
+    eval "require \"PDF/API2/Resource/CIDFont/CMap/$fname.cmap\"";
     unless($@){
         return({%{$cmap->{$fname}}});
     } else {
@@ -338,6 +298,57 @@ sub readcffdict
     return($dict);
 }
 
+sub read_kern_table
+{
+    my $font=shift @_;
+    my $upem=shift @_;
+    my $self=shift @_;
+    my $fh=$font->{' INFILE'};
+    my $data=undef;
+
+    return(undef) unless($font->{kern});
+
+    my $buf = undef;
+
+    seek($fh,$font->{kern}->{' OFFSET'}+2,0);
+    read($fh,$buf, 2);
+    my $num=unpack('n',$buf);
+    foreach my $n (1..$num)
+    {
+        read($fh, $buf, 6);
+        my ($ver, $len, $cov) = unpack('n3', $buf);
+        $len-=6;
+        my $fmt=$cov>>8;
+        if($fmt==0)
+        {
+            $data||={};
+            read($fh, $buf, 8);
+            my $nc = unpack('n', $buf);
+            foreach (1..$nc)
+            {
+                read($fh, $buf, 6);
+                my ($idx1,$idx2,$val)=unpack('nnn',$buf);
+                $val-=65536 if($val>32767);
+                $val= $val < 0 ? -floor($val*1000/$upem) : -ceil($val*1000/$upem);
+                if($val != 0)
+                {
+                    $data->{"$idx1:$idx2"}=$val;
+                    $data->{$self->data->{g2n}->[$idx1].':'.$self->data->{g2n}->[$idx2]}=$val;
+                }
+            }
+        }
+        elsif($fmt==2)
+        {
+            read($fh, $buf, $len);
+        }
+        else
+        {
+            read($fh, $buf, $len);
+        }
+    }
+    return($data);
+}
+
 sub readcffstructs
 {
     my $font=shift @_;
@@ -407,11 +418,11 @@ sub readcffstructs
 }
 
 sub new {
-    my ($class,$pdf,$file)=@_;
+    my ($class,$pdf,$file,%opts)=@_;
     my $data={};
 
     die "cannot find font '$file' ..." unless(-f $file);
-    my $font=PDF::API2::Basic::TTF::Font->open($file);
+    my $font=Font::TTF::Font->open($file);
     $data->{obj}=$font;
 
     $class = ref $class if ref $class;
@@ -421,6 +432,7 @@ sub new {
     $self->{' font'}=$font;
     $self->{' data'}=$data;
 
+	$data->{noembed} = $opts{-noembed}==1 ? 1 : 0;
     $data->{iscff} = (defined $font->{'CFF '}) ? 1 : 0;
 
     $self->{Subtype}=PDFName('Type1C') if($data->{iscff});
@@ -453,7 +465,7 @@ sub new {
     }
     
     $data->{apiname}=$data->{fontname};
-    $data->{apiname}=~s/[^A-Za-z0-9]+/ /og;
+	$data->{apiname}=~s/[^A-Za-z0-9]+/ /og;
     $data->{apiname}=join('',map { $_=~s/[^A-Za-z0-9]+//og; $_=ucfirst(lc(substr($_,0,2))); $_; } split(/\s+/,$data->{apiname}));
     $data->{fontname}=~s/[\x00-\x1f\s]//og;
 
@@ -463,9 +475,10 @@ sub new {
     $data->{subname}=$font->{'name'}->find_name(2);
     $data->{subname}=~s/[\x00-\x1f\s]//og;
 
-    if(defined $font->{cmap}->read->find_ms) 
+    $font->{cmap}->read->find_ms($opts{-isocmap}||0);
+    if(defined $font->{cmap}->find_ms) 
     {
-        $data->{issymbol} = ($font->{cmap}->read->find_ms->{'Platform'} == 3 && $font->{cmap}->read->find_ms->{'Encoding'} == 0) || 0;
+        $data->{issymbol} = ($font->{cmap}->find_ms->{'Platform'} == 3 && $font->{cmap}->read->find_ms->{'Encoding'} == 0) || 0;
     } 
     else 
     {
@@ -499,19 +512,26 @@ sub new {
     $data->{capheight}=$font->{'OS/2'}->{CapHeight} || int($data->{fontbbox}->[3]*0.8);
     $data->{xheight}=$font->{'OS/2'}->{xHeight} || int($data->{fontbbox}->[3]*0.4);
 
-    if($data->{issymbol}) {
+    if($data->{issymbol}) 
+    {
         $data->{e2u}=[0xf000 .. 0xf0ff];
-    } else {
+    } 
+    else 
+    {
         $data->{e2u}=[ unpack('U*',decode('cp1252', pack('C*',0..255))) ];
     }
 
-    if(($font->{'post'}->read->{FormatType} == 3) && defined($font->{cmap}->read->find_ms)) {
+    if(($font->{'post'}->read->{FormatType} == 3) && defined($font->{cmap}->read->find_ms)) 
+    {
         $data->{g2n} = [];
-        foreach my $u (sort {$a<=>$b} keys %{$font->{cmap}->read->find_ms->{val}}) {
+        foreach my $u (sort {$a<=>$b} keys %{$font->{cmap}->read->find_ms->{val}}) 
+        {
             my $n=nameByUni($u);
             $data->{g2n}->[$font->{cmap}->read->find_ms->{val}->{$u}]=$n;
         }
-    } else {
+    } 
+    else 
+    {
         $data->{g2n} = [ map { $_ || '.notdef' } @{$font->{'post'}->read->{'VAL'}} ];
     }
 
@@ -540,17 +560,18 @@ sub new {
     else
     {
         $data->{u2g} = {};
-        my $g=0;
-        foreach my $u ($font->{'cmap'}->read->reverse) {
+
+        my $gmap=$font->{cmap}->read->find_ms->{val};
+        foreach my $u (sort {$a<=>$b} keys %{$gmap}) 
+        {
             my $uni=$u||0;
-        #    print STDERR "got g=$g u=$uni\n";
-            $data->{u2g}->{$uni}=$g;
-            $g++;
+            $data->{u2g}->{$uni}=$gmap->{$uni};
         }
         $data->{g2u}=[ map { $_ || 0 } $font->{'cmap'}->read->reverse ];
     }
 
-    if($data->{issymbol}) {
+    if($data->{issymbol}) 
+    {
         map { $data->{u2g}->{$_} ||= $font->{'cmap'}->read->ms_lookup($_) } (0xf000 .. 0xf0ff);
         map { $data->{u2g}->{$_ & 0xff} ||= $font->{'cmap'}->read->ms_lookup($_) } (0xf000 .. 0xf0ff);
     }
@@ -559,17 +580,31 @@ sub new {
 
     $data->{e2g}=[ map { $data->{u2g}->{$_ || 0} || 0 } @{$data->{e2u}} ];
     $data->{u2e}={};
-    foreach my $n (reverse 0..255) {
+    foreach my $n (reverse 0..255) 
+    {
         $data->{u2e}->{$data->{e2u}->[$n]}=$n unless(defined $data->{u2e}->{$data->{e2u}->[$n]});
     }
 
     $data->{u2n}={ map { $data->{g2u}->[$_] => $data->{g2n}->[$_] } (0 .. (scalar @{$data->{g2u}} -1)) };
 
     $data->{wx}=[];
-    foreach my $w (0..(scalar @{$data->{g2u}}-1)) {
+    foreach my $w (0..(scalar @{$data->{g2u}}-1)) 
+    {
         $data->{wx}->[$w]=int($font->{'hmtx'}->read->{'advance'}[$w]*1000/$data->{upem})
             || $data->{missingwidth};
     }
+    
+    $data->{kern}=read_kern_table($font,$data->{upem},$self);
+    delete $data->{kern} unless(defined $data->{kern});
+
+    $data->{fontname}=~s/\s+//og;
+    $data->{fontfamily}=~s/\s+//og;
+    $data->{apiname}=~s/\s+//og;
+    $data->{altname}=~s/\s+//og;
+    $data->{subname}=~s/\s+//og;
+
+    $self->subsetByCId(0);
+    
     return($self,$data);
 }
 
@@ -577,7 +612,17 @@ sub font { return( $_[0]->{' font'} ); }
 sub data { return( $_[0]->{' data'} ); }
 sub iscff { return( $_[0]->data->{iscff} ); }
 
-sub subsetByCId {
+sub haveKernPairs { return( $_[0]->data->{kern} ? 1 : 0 ); }
+
+sub kernPairCid
+{
+    my ($self, $i1, $i2) = @_;
+    return(0) if($i1==0 || $i2==0);
+    return($self->data->{kern}->{"$i1:$i2"} || 0);
+}
+
+sub subsetByCId 
+{
     my $self = shift @_;
     my $g = shift @_;
     $self->data->{subset}=1;
@@ -618,10 +663,16 @@ sub outobjdeep {
             }
         }
 
-        $self->{' stream'} = "";
-        my $ffh = PDF::API2::IOString->new(\$self->{' stream'});
-        $f->out($ffh, 'cmap', 'cvt ', 'fpgm', 'glyf', 'head', 'hhea', 'hmtx', 'loca', 'maxp', 'prep');
-        $self->{'Length1'}=PDFNum(length($self->{' stream'}));
+		if($self->data->{noembed} != 1)
+		{
+        	$self->{' stream'} = "";
+        	my $ffh;
+        	CORE::open($ffh, '+>', \$self->{' stream'});
+        	binmode($ffh,':raw');
+        	$f->out($ffh, 'cmap', 'cvt ', 'fpgm', 'glyf', 'head', 'hhea', 'hmtx', 'loca', 'maxp', 'prep');
+        	$self->{'Length1'}=PDFNum(length($self->{' stream'}));
+        	CORE::close($ffh);
+		}
     }
 
     $self->SUPER::outobjdeep($fh, $pdf, %opts);
@@ -629,60 +680,3 @@ sub outobjdeep {
 
 
 1;
-
-__END__
-
-=head1 AUTHOR
-
-alfred reibenschuh
-
-=head1 HISTORY
-
-    $Log: FontFile.pm,v $
-    Revision 1.14  2005/03/20 23:24:23  fredo
-    added workaround for broken truetype naming-tables
-
-    Revision 1.13  2005/03/14 22:01:27  fredo
-    upd 2005
-
-    Revision 1.12  2005/01/21 10:00:43  fredo
-    added handling of nosubset option
-
-    Revision 1.11  2004/12/29 01:12:27  fredo
-    fixed no warn for recursion
-
-    Revision 1.10  2004/12/16 00:30:53  fredo
-    added no warn for recursion
-
-    Revision 1.9  2004/11/22 21:07:55  fredo
-    fixed multibyte-encoding support to work consistently acress cjk/ttf/otf
-
-    Revision 1.8  2004/11/22 02:25:05  fredo
-    added advanced attributes
-
-    Revision 1.7  2004/08/25 03:03:27  fredo
-    removed fuss
-
-    Revision 1.6  2004/06/15 09:14:52  fredo
-    removed cr+lf
-
-    Revision 1.5  2004/06/07 19:44:43  fredo
-    cleaned out cr+lf for lf
-
-    Revision 1.4  2004/04/20 09:46:25  fredo
-    added glyph->read fix for subset-vector
-
-    Revision 1.3  2003/12/08 13:06:01  Administrator
-    corrected to proper licencing statement
-
-    Revision 1.2  2003/11/30 17:31:41  Administrator
-    merged into default
-
-    Revision 1.1.1.1.2.2  2003/11/30 16:57:02  Administrator
-    merged into default
-
-    Revision 1.1.1.1.2.1  2003/11/30 14:16:39  Administrator
-    added CVS id/log
-
-
-=cut
