@@ -1,6 +1,6 @@
 package PDF::API2::Resource::ColorSpace::Indexed::ACTFile;
 
-our $VERSION = '2.019';
+our $VERSION = '2.020'; # VERSION
 
 use base 'PDF::API2::Resource::ColorSpace::Indexed';
 
@@ -58,30 +58,4 @@ sub new {
     return($self);
 }
 
-=item $cs = PDF::API2::Resource::ColorSpace::Indexed->new_api $api, $name
-
-Returns a indexed color-space object. This method is different from 'new' that
-it needs an PDF::API2-object rather than a Text::PDF::File-object.
-
-=cut
-
-sub new_api {
-    my ($class,$api,@opts)=@_;
-
-    my $obj=$class->new($api->{pdf},@opts);
-    $self->{' api'}=$api;
-
-    return($obj);
-}
-
 1;
-
-__END__
-
-=back
-
-=head1 AUTHOR
-
-Alfred Reibenschuh
-
-=cut
